#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Tech - JSON - XML
# 
# Transmettre l'information d'une machine à une autre, d'un logiciel à un autre, d'une base de données à une autre est un problème récurrent. Le format le plus simple pour des données est le format [csv](https://fr.wikipedia.org/wiki/Comma-separated_values). Ca marche bien pour les tables mais cela ne permet de transmettre aisément des [données non structurées](https://fr.wikipedia.org/wiki/Informations_non_structur%C3%A9es).

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Enoncé

# In[2]:


from sklearn.datasets import load_iris as load_data
from pandas import DataFrame
data = load_data()
df = DataFrame(data.data, columns=data.feature_names)
df['fleur'] = [data.target_names[t] for t in data.target]
df.tail()


# ### Q1 : écriture des données au format CSV
# 
# Le plus simple...

# In[3]:


from io import StringIO
buffer = StringIO()
df.to_csv(buffer, index=False)
text = buffer.getvalue()
text[:300]


# In[4]:





# ### Q2 : écriture des données au format JSON

# In[5]:


r = df.to_json(orient='records')
r[:400]


# In[6]:





# ### Q3 : relire les données avec le module [json](https://docs.python.org/3/library/json.html)

# In[7]:





# In[8]:





# ### Q4 : essayez avec les format XML (ou HTML), SQL, SAS, Excel...

# In[9]:





# ### Q5 : données non structurées
# 
# On ajoute les endroits où ces fleurs sont présentes. On voudrait que toutes les informations soient présentes dans le même fichier. Comment fait-on ?

# In[10]:


locations = {'virginica': ['Florida', 'Georgia'],
             'setosa': ['Maine', 'Alaska', 'Quebec'],
             'versicolor': ['Quebec', 'Georgia', 'Ireland', 'Main']}


# La question sous-jacente est : vaut-il mieux avoir deux fichiers plats, l'un pour les données décrivant les fleurs, l'autre pour les localisations ou un seul fusionnant les deux informations ?

# In[11]:





# ### Q6 : le texte, ça prend trop de place, zippons
# 
# Avec le module [zipfile](https://docs.python.org/3/library/zipfile.html).

# In[12]:





# ### Q7 : que vous inspire [protobuf](https://developers.google.com/protocol-buffers) ?
# 
# Ou [flatbuffers](https://google.github.io/flatbuffers/), [MessagePack](https://msgpack.org/)

# In[13]:





# ## Réponses

# ### Q1 : écriture des données au format CSV

# In[14]:


from io import StringIO
buffer = StringIO()
df.to_csv(buffer, index=False)
text = buffer.getvalue()
text[:300]


# In[15]:


df.to_csv("fleurs.csv", index=False)


# In[16]:


import os
os.listdir(".")


# In[17]:


import pandas
df2 = pandas.read_csv("fleurs.csv")


# In[18]:


df2.head()


# In[19]:


virtuel = StringIO(text)
df3 = pandas.read_csv(virtuel)
df3.head()


# ### Q2 : écriture des données au format JSON

# In[20]:


json_text = df.to_json(orient='records')
json_text[:400]


# ### Q3 : relire les données avec le module [json](https://docs.python.org/3/library/json.html)

# In[21]:


import json


# In[22]:


res = json.loads(json_text)


# In[23]:


for i, r in enumerate(res):
    print(i, type(r), r)
    if i >= 5:
        break


# In[24]:


res[3]['sepal width (cm)']


# In[25]:


virtuel = StringIO(json_text)
res2 = json.load(virtuel)
res2[:3]


# ### Q4 : essayez avec les format XML (ou HTML), SQL, SAS, Excel...

# In[26]:


html_text = df.to_html(index=False)


# In[27]:


print(html_text[:500])


# In[28]:


df_html = pandas.read_html(html_text)
df_html[0].tail()


# In[29]:


df_html = pandas.read_html(html_text + html_text)
len(df_html)


# ### Q5 : données non structurées

# In[30]:


df.head()


# In[31]:


locations = {'virginica': ['Florida', 'Georgia'],
             'setosa': ['Maine', 'Alaska', 'Quebec'],
             'versicolor': ['Quebec', 'Georgia', 'Ireland', 'Main']}


# In[32]:


obs = []
for fleur, loc in locations.items():
    for l in loc:
        obs.append({"fleur": fleur, "location": l})
obs


# In[33]:


df_locations = pandas.DataFrame(obs)
df_locations


# In[34]:


merged = df.merge(df_locations, left_on="fleur", right_on="fleur")
merged.head(10)


# In[35]:


merged.shape


# In[36]:


locations


# In[37]:


obs2 = []
for fleur, loc in locations.items():
    obs2.append({"fleur": fleur, "location": loc})
obs2


# In[38]:


df_locations2 = pandas.DataFrame(obs2)
df_locations2


# In[39]:


merged = df.merge(df_locations2, left_on="fleur", right_on="fleur")
merged.head(10)


# In[40]:


json_text = merged.to_json(orient='records')
json_text[:200]


# In[41]:


df.to_excel("data.xlsx", index=False)


# In[42]:


dfe = pandas.read_excel("data.xlsx", engine='openpyxl')
dfe.tail()


# ### Q6 : le texte, ça prend trop de place, zippons

# In[43]:


from zipfile import ZipFile

with ZipFile('data.zip', 'w') as myzip:
    myzip.write('data.xlsx')
    myzip.write("2020_json_xml.ipynb")


# In[44]:


import glob
glob.glob("*.zip")


# ### Q7 : que vous inspire [protobuf](https://developers.google.com/protocol-buffers) ?
# 
# Protobuf est un format de sérialisation au même titre que [pickle](https://docs.python.org/3/library/pickle.html). La [sérialisation](https://fr.wikipedia.org/wiki/S%C3%A9rialisation) désigne un procédé qui permet d'enregister tout un tas d'information sous la forme d'ensemble d'octets contigü. En gros, on enregistre un ensemble de variables en même quel qu'il soit, dans un unique fichier contigü. [json](https://fr.wikipedia.org/wiki/JavaScript_Object_Notation) est un format utilisé par les outils de sérialisation. Tout est en texte et la plupart du temps, les nombres réelles prennent moins de place quand ils sont stockés comme ils sont sockés en mémoire sur 8 octets. C'est le premier problème résolu par protobuf. Ensuite, le format *json* est très générique mais il suggère de stocker le nom des colonnes à chaque ligne, c'est une information qui est dupliqué à chaque ligne... *protobuf* propose une façon de ne pas les sotcker du tout. Je passe les détails. Ils reviendront quand le problème de communiquer efficacement des données se posera.

# In[45]: